@@ -114,6 +114,35 @@ def _to_api_repr_table_defs(value):
     return {k: ExternalConfig.to_api_repr(v) for k, v in value.items()}
 
 
+class DmlStats(typing.NamedTuple):
+    """Detailed statistics for DML statements.
+
+    https://cloud.google.com/bigquery/docs/reference/rest/v2/DmlStats
+    """
+
+    inserted_row_count: int = 0
+    """Number of inserted rows. Populated by DML INSERT and MERGE statements."""
+
+    deleted_row_count: int = 0
+    """Number of deleted rows. populated by DML DELETE, MERGE and TRUNCATE statements.
+    """
+
+    updated_row_count: int = 0
+    """Number of updated rows. Populated by DML UPDATE and MERGE statements."""
+
+    @classmethod
+    def from_api_repr(cls, stats: Dict[str, str]) -> "DmlStats":
+        # NOTE: The field order here must match the order of fields set at the
+        # class level.
+        api_fields = ("insertedRowCount", "deletedRowCount", "updatedRowCount")
+
+        args = (
+            int(stats.get(api_field, default_val))
+            for api_field, default_val in zip(api_fields, cls.__new__.__defaults__)
+        )
+        return cls(*args)
+
+
 class ScriptOptions:
     """Options controlling the execution of scripts.
 
@@ -1079,6 +1108,14 @@ class QueryJob(_AsyncJob):
             result = int(result)
         return result
 
+    @property
+    def dml_stats(self) -> Optional[DmlStats]:
+        stats = self._job_statistics().get("dmlStats")
+        if stats is None:
+            return None
+        else:
+            return DmlStats.from_api_repr(stats)
+
     def _blocking_poll(self, timeout=None, **kwargs):
         self._done_timeout = timeout
         self._transport_timeout = timeout
@@ -1349,12 +1386,12 @@ class QueryJob(_AsyncJob):
 
                 This argument does nothing if ``bqstorage_client`` is supplied.
 
-                ..versionadded:: 1.24.0
+                .. versionadded:: 1.24.0
 
             max_results (Optional[int]):
                 Maximum number of rows to include in the result. No limit by default.
 
-                ..versionadded:: 2.21.0
+                .. versionadded:: 2.21.0
 
         Returns:
             pyarrow.Table
@@ -1366,7 +1403,7 @@ class QueryJob(_AsyncJob):
             ValueError:
                 If the :mod:`pyarrow` library cannot be imported.
 
-        ..versionadded:: 1.17.0
+        .. versionadded:: 1.17.0
         """
         query_result = wait_for_query(self, progress_bar_type, max_results=max_results)
         return query_result.to_arrow(
@@ -1415,7 +1452,7 @@ class QueryJob(_AsyncJob):
                 :func:`~google.cloud.bigquery.table.RowIterator.to_dataframe`
                 for details.
 
-                ..versionadded:: 1.11.0
+                .. versionadded:: 1.11.0
             create_bqstorage_client (Optional[bool]):
                 If ``True`` (default), create a BigQuery Storage API client
                 using the default API settings. The BigQuery Storage API
@@ -1424,18 +1461,18 @@ class QueryJob(_AsyncJob):
 
                 This argument does nothing if ``bqstorage_client`` is supplied.
 
-                ..versionadded:: 1.24.0
+                .. versionadded:: 1.24.0
 
             date_as_object (Optional[bool]):
                 If ``True`` (default), cast dates to objects. If ``False``, convert
                 to datetime64[ns] dtype.
 
-                ..versionadded:: 1.26.0
+                .. versionadded:: 1.26.0
 
             max_results (Optional[int]):
                 Maximum number of rows to include in the result. No limit by default.
 
-                ..versionadded:: 2.21.0
+                .. versionadded:: 2.21.0
 
         Returns:
             A :class:`~pandas.DataFrame` populated with row data and column
